*! version 4.0.7  30aug2000 
program define alpha, rclass
	version 6.0
	
	syntax varlist(min=2 numeric) [if] [in] [, Asis Casewise Coo Detail Item /*
		*/ Generate(string) Label Min(integer 1) Reverse(varlist numeric) Std ]

	tokenize `varlist'
	local k : word count `varlist'

	if `min' < 1 | `min' > `k' {
		di in re "invalid option min()"
		exit 198
	}

	if `k' == 2 & "`item'" ~= "" {
		di in bl "option -item- ignored with 2 variables"
		local item
	}
	
	* scratch
  
	tempvar cnt nobs rest test touse x 
	tempname ac acor acorr alph alpha av avar C mIIC MEAN nc ncor nv nvar Obs IRC ITC VAR

/* begin */ quietly { 

	* select and count cases; treatment of missing values 
	*   N = #obs selected via if/in/min
	*  NC = #complete obs 

	mark `touse' `if' `in'
	if "`coo'" != "" { 
		local casewis casewise 
	}
	if "`casewis'" == "" & `min' < `k' {      /* incomplete cases allowed */
	
		egen `nobs' = robs(`varlist') if `touse'
		replace `touse' = (`nobs' != . & `nobs' >= `min')
		count if `touse'
		local N = r(N)		/* # if in */
		count if `nobs'==`k' & `touse'
		local NC = r(N)   /* # if in & complete */
	}
	else {                                   /* only complete cases */    
	
		markout `touse' `varlist'
		count if `touse'
		local N = r(N)
		local NC `N'
	}    

	* signs of variables sgn1, sgn2, ... 

	local i 1
	while `i' <= `k' {
		local sgn`i' 1
		local i = `i' + 1
	}

	if `"`reverse'"' != "" {
		tokenize `reverse'
		local revlist
		while "`1'" != "" {
			ListIndx "`varlist'" `"`1'"'
			if $S_1 > 0  { 
				local sgn$S_1 -1 
				local revlist "`revlist' `1'"
			}
			else noisily di in bl "`1' is not included in varlist"
			mac shift
		}
		* the code below assumes parsing of the varlist
		tokenize `varlist'
	}
	
	else if "`asis'" == "" { 
	   tempvar score
		factor `varlist' if `touse'
		score `score'
		local i 1
		while `i' <= `k' {
			corr ``i'' `score' if `touse'
			if r(rho) < 0 { 
				local revlist "`revlist' ``i''"
				local sgn`i' -1
			}
			local i = `i' + 1
		}
	}
    
	* matrix C    = corr/cov for signed variables 
	* matrix Obs  = pairwise number of non-missing cases  
	* matrix MEAN = unsigned means of variables 
	* matrix VAR  = variance of variables 
	* var cnt     = #non-mv's

	if `N' != `NC' {                    /* some cases contain missing values */

		mat `Obs'  = J(`k',`k',0)       
		mat `C'    = `Obs'              
		mat `MEAN' = J(1,`k',0)         
		mat `VAR'  = J(1,`k',0)         
		gen int `cnt' = 0               

		if "`std'" == "" { 
		   local corropt  cov  
			local r_arg    cov_12
		}
		else	local r_arg rho
		
		local i 1
		while `i' <= `k' {
			summ ``i'' if `touse'
			mat `Obs'[`i',`i'] = `r(N)'
			mat `MEAN'[1,`i']  = `r(mean)'
			mat `VAR'[1,`i']   = `r(Var)'
			if "`std'" == "" { mat `C'[`i',`i'] = `r(Var)' }
			else               mat `C'[`i',`i'] = 1 
			replace `cnt' = `cnt' + 1 if `touse' & ``i''!=.

			local j 1
 			while `j' < `i' {
				corr ``i'' ``j'' if `touse', `corropt'
				if r(N) != 0 {
					mat `C'[`i',`j'] = `sgn`i'' * `sgn`j'' * r(`r_arg')
					mat `C'[`j',`i'] = `C'[`i',`j'] 
				}
				mat `Obs'[`i',`j'] = `r(N)'
				mat `Obs'[`j',`i'] = `Obs'[`i',`j'] 
				local j = `j' + 1
			}
			local i = `i' + 1
		} /* while i */
	}
	else {                                     /* all cases complete */

		mat acc `C' = `varlist' if `touse', dev mean(`MEAN') nocons
		mat `C' = (1/(r(N)-1)) * `C'
		mat `VAR' = vecdiag(`C')
        
		* adapt C for the signs of variables
		if "`revlist'" != "" {
			local i 1
			while `i' <= `k' {
				local j 1
				while `j' < `i' {
					mat `C'[`i',`j'] = `C'[`i',`j'] * `sgn`i'' * `sgn`j''
					mat `C'[`j',`i'] = `C'[`i',`j'] 
					local j = `j' + 1
				}
				local i = `i' + 1
			}
		}

		* modifcation in case of correlations 
		if "`std'" != "" { mat `C' = corr(`C') }
		mat `Obs' = J(`k',`k',`N')
		gen int `cnt' = `k'
	}    

	* test scores

	gen `test' = 0 if `touse'      
	local i 1
	while `i' <= `k' {
		if "`std'" == "" {
			replace `test' = `test' + `sgn`i'' * ``i'' if `touse' & ``i''!=.
		}
		else replace `test' = `test' + /*
			*/  `sgn`i'' * (``i''-`MEAN'[1,`i']) / sqrt(`VAR'[1,`i']) /*
			*/  if `touse' & ``i''!=.
		local i = `i' + 1
	}
	* note: at this point, cnt>0 is ensured
	replace `test' = `test' / `cnt' if `touse'

	* compute statistics for the additive scale in all items

	if "`std'" != "" { 
		scalar `avar' = 1
		scalar `nvar' = 1
	}
	else {
		scalar `avar' = 0
		scalar `nvar' = 0
		local i 1
		while `i' <= `k' {
			scalar `avar' = `avar' + `Obs'[`i',`i'] * `C'[`i',`i']
			scalar `nvar' = `nvar' + `Obs'[`i',`i']
			local i = `i' + 1
		}
	}

	scalar `acor' = 0                         /* weighted sum of variances */
	scalar `ncor' = 0                         /* sum of weights */
	local i 1
	while `i' <= `k' {
		local j 1
		while `j' < `i' {
			scalar `acor' = `acor' + `Obs'[`i',`j'] * `C'[`i',`j']
			scalar `ncor' = `ncor' + `Obs'[`i',`j']        
			local j = `j' + 1
		}
		local i = `i' + 1
	}
    
	scalar `acorr' = `acor'/`ncor'
   scalar `alpha' = (`k'*`acorr')/(`avar'/`nvar'+(`k'-1)*`acorr')
	if `alpha' < 0 { scalar `alpha' = . }

	* double save in r() and S_# 
 	if "`std'" == "" { ret scalar cov = `acorr' }
	else             { ret scalar rho = `acorr' }
	ret scalar k = `k'
	ret scalar alpha = `alpha'
	
	global S_4 = scalar(`acorr')
	global S_5 = `k'
	global S_6 = scalar(`alpha')

	* compute statistics for additive scales formed by removing single items

	if "`item'" != "" {

		mat `ITC'  = J(1,`k',0)          /* item-test correlations */
		mat `IRC'  = J(1,`k',0)          /* item-rest correlations */
		mat `mIIC' = J(1,`k',0)          /* mean InterItem corr/cov */
		mat `alph' = J(1,`k',0)          /* alpha */

		if "`std'" != "" {
			scalar `av' = 1        
			scalar `nv' = 1
		}
		local i 1
		while `i' <= `k' {
			scalar `ac' = `acor'
			scalar `nc' = `ncor'
			local j 1
			while `j' <= `k' {
				if `i' != `j' {
					scalar `ac' = `ac' - `Obs'[`i',`j'] * `C'[`i',`j']
					scalar `nc' = `nc' - `Obs'[`i',`j']        
				}
				local j = `j' + 1
			}
			if "`std'"=="" {
				scalar `av' = `avar' - `Obs'[`i',`i'] * `C'[`i',`i']
				scalar `nv' = `nvar' - `Obs'[`i',`i']
			}
			mat `mIIC'[1,`i'] = `ac'/`nc'
			mat `alph'[1,`i'] = /*
				*/ ((`k'-1)*`ac'/`nc') / ((`av'/`nv')+(`k'-2)*`ac'/`nc')
    
			* Item-Test correlations 
			corr ``i'' `test' if `touse'
			mat `ITC'[1,`i'] = `sgn`i'' * r(rho) 

			* Item-Rest (=Test-Item) correlations 
			if "`std'" == "" {
				gen `x' = `sgn`i'' * ``i'' if `touse' & ``i''!=.
			}        
			else gen `x' = `sgn`i''*(``i''-`MEAN'[1,`i'])/sqrt(`VAR'[1,`i']) /*
				*/ if `touse' & ``i''!=.
					    
			gen `rest' = (`cnt'*`test'-`x') / (`cnt'-1) /*
					*/ if `touse' & ``i''!=. & `cnt'>1
			corr ``i'' `rest' if `touse'
			mat `IRC'[1,`i'] = `sgn`i'' * r(rho)
			drop `x' `rest'    

			local i = `i' + 1
		}
	}
 
/* end quietly */ } 

	* --------------------------------------------------------------------------
	* display   
	* --------------------------------------------------------------------------

	if "`std'" == "" { local un "un" }
	di _n in gr "Test scale = mean(`un'standardized items)"

	* compact output 

	if "`item'" == "" {       
		if "`std'" == "" { 
			local lab "covariance"
			local fmt "%9.0g"
		}
		else {
			local lab "correlation"
			local fmt "%9.4f"
		}        
 		if "`revlist'" != "" {
		   local nrev : word count `revlist'
			local itemtxt = cond(`nrev'>1, "items", "item")
			di in gr "Reversed `itemtxt':" in ye "`revlist'"
		}
		di
		di in gr "Average interitem `lab':"       _col(34) in ye `fmt' `acorr'
		di in gr "Number of items in the scale:"  _col(34) in ye %9.0f `k'
		di in gr "Scale Reliability Coefficient:" _col(34) in ye %9.4f `alpha'
	}    

	* wide detailed output 

	if "`item'" != "" & "`label'" == "" {        
		if "`std'" == "" { 
			local lab "covariance "
			local fmt "%9.0g"
		}
		else {
			local lab "correlation"
			local fmt "%8.4f"
		}        
    
		di _n in gr "                         item-test     item-rest     " /*
			*/ " interitem"
		di    in gr "Item     |  Obs  Sign   correlation   correlation    " /*
			*/ "`lab'      alpha"   
		di    in gr "---------+" _dup(68) "-"
		local i 1
		while `i' <= `k' {
			if "`sgn`i''" == "1" { local c "+" }
			else                 { local c "-" }
			di in gr "``i''"  _col(10) "|" in ye  /*
				*/ _col(12) %4.0f `Obs'[`i',`i'] /*
				*/ _col(20)       "`c'"          /*
				*/ _col(27) %7.4f `ITC'[1,`i']   /*
				*/ _col(41) %7.4f `IRC'[1,`i']   /*
				*/ _col(55) `fmt' `mIIC'[1,`i']  /*
				*/ _col(69) %7.4f `alph'[1,`i'] 
			local i = `i' + 1
		}
		di in gr "---------+" _dup(68) "-"
		di in gr "Test" in ye _col(55) `fmt' `acorr' _col(69) %7.4f `alpha' 
		di in gr "---------+" _dup(68) "-"
	}

	* compact detailed & labeled output 

	if "`item'" != "" & "`label'" != "" { 
		if "`std'" == "" { 
			local lab "cov"    
			local fmt "%7.3g" 
		}
		else {
			local lab "cor"    
			local fmt "%7.3f" 
		}

		di _n in gr "Items    | S  it-cor  ir-cor   ii-`lab'   alpha   label"
		di    in gr "---------+" _dup(68) "-"
		local i 1
		while `i' <= `k' {
         if "`sgn`i''" == "1" { local c "+" }
			else local c "-" 
			
			local vlab : var label ``i''
			local vl : piece 1 29 of `"`vlab'"'
			
			di in gr "``i''"  _col(10) "| " in ye "`c'" /*
				*/ _col(15) %6.3f `ITC'[1,`i']   /*
				*/ _col(23) %6.3f `IRC'[1,`i']   /*
				*/ _col(31) `fmt' `mIIC'[1,`i']  /*
				*/ _col(40) %6.3f `alph'[1,`i']  /*
				*/ _col(49) in gr `"`vl'"'				

         local j 2			
			local vl : piece `j' 29 of `"`vlab'"'
			while `"`vl'"' ~= "" {
            di in gr _col(10) "|" _col(51) `"`vl'"' 
				local j = `j'+1			
            local vl : piece `j' 29 of `"`vlab'"'
         }				
			
			local i = `i' + 1
		}
		di in gr "---------+" _dup(68) "-"
		di in gr "Test" in ye _col(31) `fmt' `acorr' _col(40) %6.3f `alpha' /*
			*/ _col(49) in gr "mean(`un'standardized items)"
		di in gr "---------+" _dup(68) "-"
	}        

	* --------------------------------------------------------------------------
	* detail: table with interitem correlations / covariances  
	* --------------------------------------------------------------------------

	if "`detail'" != "" {
		
		if `N' == `NC' { local o "(obs=`N' in all pairs)" }
		else           { local o "(obs=pairwise, see below)" }
		
		if "`revlist'" ~= "" {
			local r "(reverse applied) "
		}	
		
		if "`std'" != "" {    
			di _n in gr "Interitem correlations `r'`o'"
			mat list `C', noheader format(%6.4f)
		}
		else {
			di _n in gr "Interitem covariances `r'`o'"
			mat list `C', noheader format(%9.4g)
		}
		
		if `N' != `NC' {
			di _n in gr "Pairwise number of observations"
			mat rowname `Obs' = `varlist'
			mat colname `Obs' = `varlist'
			mat list `Obs', noheader format(%5.0f)
		}    
	}

   * --------------------------------------------------------------------------
	* generate(): save test scale 
	* --------------------------------------------------------------------------

	if "`generat'" != "" {
		confirm new var `generat'
		qui gen `generat' = `test' if `touse'
		label var `generat' "mean(`un'standardized items)"
	}   
end

* -ListIndx list word- returns the index of word in list, or 0 if not found
program define ListIndx
	args list word
	
	local word = trim(`"`word'"')
	tokenize `list'
	local i 1
	local ifound 0
	while `"``i''"' != ""  & `ifound' == 0 {
		if `"``i''"' == `"`word'"' { local ifound `i' }
		local i = `i'+1   
	}    
	global S_1 `ifound'
end
exit


HISTORY
4.0.4  improved varlabel handling in with option -label

4.0.3  added option numeric to varlist
       improved handling of long variable labels, and labels than contain quotes
       bug fix: -nvar=2, item-
       used matrix expressions
       modified display of signs in detail tables (the old version would crash if
         the name of a reversed variable was 8 char

comments for help file
  #variables in varlist >=2
  #variables in reverse() not included in varlist are ignored
  item is *ignored* only with >2 variables.
  


LAYOUT-1

         1         2         3         4         5         6         7
1234567890123456789012345678901234567890123456789012345678901234567890123456789


                         item-test     item-rest     interitem
Item     |  Obs  Sign   correlation   correlation    correlation      alpha   
---------+-----------------------------------------------------------------
12345678 | 1234    -       0.0000        0.0000       0000.0000      0.0000   
12345678 |         -       0.0000        0.0000         0.0000       0.0000   
---------+------------------------------------------------------------------
all items                                             0000.0000      0.0000   


LAYOUT-2 

         1         2         3         4         5         6         7
1234567890123456789012345678901234567890123456789012345678901234567890123456789

Items    | S  it-cor  ir-cor  aii-cor   alpha   label
---------+---------------------------------------------------------------------
12345678 | -   0.000   0.000    0.000   0.000   1234567890123456789012345678901
12345678 | +   0.000   0.000    0.000   0.000   1234567890123456789012345678901
---------|---------------------------------------------------------------------
all items                       0.000   0.000   

aii = average interitem covariance/correlation  
ir  = item-rest correlations (rest = test-item)
it  = item-test correlations 

Future developments
 * We could, of course, compute the item-test and item-test correlations
   from the correlation/covariance matrix C, rather than from the raw data.
   Given Stata's limited matrix language, this is no fun.
 * Bartlet's test for homogeneity of variances
 * ...
