*! version 6.0.2  18aug1999
program define gamma_p
	version 6

		/* Step 1:
			place command-unique options in local myopts
			Note that standard options are
			LR:
				Index XB Cooksd Hat 
				REsiduals RSTAndard RSTUdent
				STDF STDP STDR noOFFset
			SE:
				Index XB STDP noOFFset
		*/
	local myopts /*
*/ "Index XB STDP Time LNTime HAzard Surv DEViance CSnell MGale CSUrv CCSnell CMgale OOS"


		/* Step 2:
			call _propts, exit if done, 
			else collect what was returned.
		*/
	_pred_se "`myopts'" `0'
	if `s(done)' { exit }
	local vtyp  `s(typ)'
	local varn `s(varn)'
	local 0 `"`s(rest)'"'


		/* Step 3:
			Parse your syntax.
		*/
	syntax [if] [in] [, `myopts' noOFFset]


		/* Step 4:
			Concatenate switch options together
		*/
	local type /*
*/ "`xb'`index'`stdp'`time'`lntime'`hazard'`surv'`deviance'`csnell'`mgale'`csurv'`ccsnell'`cmgale'"


		/* Step 5:
			quickly process default case if you can 
			Do not forget -nooffset- option.
		*/

		/* Step 6:
			mark sample (this is not e(sample)).
		*/
	marksample touse
	if "`e(cmd2)'"=="streg" { 
		st_is 2 full
		local is_st yes
		local id `_dta[st_id]'
		local t  `_dta[st_t]'
		local t0 `_dta[st_t0]'
		local d  `_dta[st_d]'
		qui replace `touse'=0 if _st==0
	}
	else {
		local t `e(depvar)'
		local t0 `e(t0)'
		local d `e(dead)'
	}



	*tempname sgn rgn
	*scalar `sgn' = cond(e(frm2)=="time", 1, -1/e(aux_p))
	*scalar `rgn' = cond(e(frm2)=="time", -e(aux_p), 1)

	if "`type'"=="" | "`type'"=="time" {
		if "`type'"=="" {
			di in gr "(option time assumed; predicted time)"
		}
		tempvar lt 
		qui predict double `lt' if `touse', lntime
		gen `vtyp' `varn'=exp(`lt') if `touse'
		label var `varn' "predicted `e(depvar)'"
		exit
	}

	if "`type'"=="xb" | "`type'"=="index" | "`type'"=="stdp" {
		_predict `vtyp' `varn' if `touse', `type' `offset'
		exit
	}

	if "`type'"=="lntime" {
		tempvar xb 
		tempname nkappa
		scalar `nkappa'=abs(e(kappa))
		qui _predict double `xb' if `touse', xb `offset'
		gen `vtyp' `varn'=`xb' /*
		*/ + e(sigma)/ `nkappa'*(digamma(`nkappa'^-2) /*
		*/ -log(`nkappa'^-2)) if `touse'
		label var `varn' "predicted ln(`e(depvar)')"
		exit
	}
/*
	if "`type'"=="hr" {
		tempvar xb
		qui _predict double `xb' if `touse', xb `offset'
		gen `vtyp' `varn'=exp(`rgn'*(`xb'-_b[_cons])) if `touse'
		label var `varn' "hazard ratio"
		exit
	}
*/

	if "`type'"=="hazard" {
		tempvar xb ff z s
		tempname sgn l
		scalar `sgn'=cond(e(kappa)<0,-1,1)		
		scalar `l'=e(kappa)^(-2)		
		qui _predict double `xb' if `touse', xb `offset'
		qui gen double `z'= `sgn'*(ln(`t')-`xb')/e(sigma)
                qui gen double `s'= gammap(`l',`l'*exp(`z'/sqrt(`l')))
                if `sgn'==1 {
                        qui replace `s'=1-`s'
                }

		qui gen double `ff'=((`l'-0.5)*ln(`l')) /*
		   */  + (`z'*sqrt(`l'))-`l'*exp(`z'/sqrt(`l')) /*
		   */ -lngamma(`l') - ln(`t'*e(sigma)) 
		qui replace `ff'= exp(`ff')

		gen `vtyp' `varn'= `ff'/`s' if `touse'
		label var `varn' "predicted hazard"
		exit
	}

	if "`type'" =="surv" {
		tempvar xb ff z z0 ff0
		tempname sgn l
		scalar `sgn'=cond(e(kappa)<0,-1,1)		
		scalar `l'=(e(kappa)^-2)		
		qui _predict double `xb' if `touse', xb `offset'
		qui gen double `z' = `sgn'*(ln(`t')-`xb')/e(sigma)
		qui gen double `z0' = `sgn'*(ln(`t0')-`xb')/e(sigma) if `t0'>0
		qui gen double `ff'= gammap(`l',`l'*exp(`z'/sqrt(`l')))
		qui gen double `ff0'= gammap(`l',`l'*exp(`z0'/sqrt(`l'))) /*
		  */ if `t0'>0
		if `sgn'==1 {
			qui replace `ff'=1-`ff'	
			qui replace `ff0'=1-`ff0' if `t0'>0
		}	
		qui replace `ff'= `ff'/`ff0' if `t0'>0

		gen `vtyp' `varn' = `ff' if `touse'
		if "`t0'"=="0" { 
			label var `varn' "S(`t')"
		}
		else	label var `varn' "S(`t'|`t0')"
		exit
	}

	if "`type'"=="csnell" {
		tempvar mg
		qui predict double `mg' if `touse', mgale `offset'
		gen `vtyp' `varn' = (`d'!=0) - `mg' if `touse'
		if "`id'"!="" { local part "partial " }
		label var `varn' "`part'Cox-Snell residual"
		exit
	}

	if "`type'"=="mgale" { 
		tempvar xb ff
		qui predict double `ff',surv
		gen `vtyp' `varn' = (`d'!=0)+ln( `ff') if `touse' 
		if "`id'"!="" { local part "partial " }
		label var `varn' "`part'Martingale-like resid."
		exit
	}



		/* Step 7:
			handle options that take argument one at a time.
			Comment if restricted to e(sample).
			Be careful in coding that number of missing values
			created is shown.
			Do all intermediate calculations in double.
		*/


		/* Step 8:
			handle switch options that can be used in-sample or 
			out-of-sample one at a time.
			Be careful in coding that number of missing values
			created is shown.
			Do all intermediate calculations in double.
		*/

	if "`is_st'"=="yes" {
		if "`type'"=="deviance" {
			tempvar cmg 
			qui predict double `cmg' if `touse', cmgale `offset' /*
				*/ `oos'
			gen `vtyp' `varn' = sign(`cmg')*sqrt( /* 
			*/ -2*(`cmg'+(`d'!=0)*(ln((`d'!=0)-`cmg')))) if `touse'
			label var `varn' "deviance residual"
			exit
		}

		if "`oos'"=="" {
			tempvar es
			qui gen byte `es' = e(sample)
		}
		else	local es "`touse'"


		if "`type'"=="csurv" {
			if "`_dta[st_id]'" == "" {
				predict `vtyp' `varn' if `touse' & `es', /*
				*/ surv `offset' 
				exit
			}
			tempvar surv
			qui predict double `surv' if `es', surv `offset'
			sort `es' `_dta[st_id]' `t'
			qui by `es' `_dta[st_id]': replace /*
				*/ `surv'=`surv'*`surv'[_n-1] if _n>1 & `es'
			gen `vtyp' `varn' = `surv' if `touse'
			label var `varn' "S(`t'|earliest `t0' for subj.)"
			exit
		}
		if "`type'"=="ccsnell" {
			if "`_dta[st_id]'" == "" {
				predict `vtyp' `varn' if `touse' & `es', /*
				*/ csnell `offset' 
				exit
			}
			tempvar cs
			qui predict double `cs' if `es', cs `offset'
			sort `es' `_dta[st_id]' `t'
			qui by `es' `_dta[st_id]': replace /*
				*/ `cs'=cond(_n==_N,sum(`cs'),.) if `es'
			gen `vtyp' `varn' = `cs' if `touse'
			label var `varn' "cum. Cox-Snell residual"
			exit
		}
		if "`type'"=="cmgale" {
			if "`_dta[st_id]'" == "" {
				predict `vtyp' `varn' if `touse' & `es', /*
				*/ mgale `offset' 
				exit
			}
			tempvar mg
			qui predict double `mg' if `es', mg `offset'
			sort `es' `_dta[st_id]' `t'
			qui by `es' `_dta[st_id]': replace /*
				*/ `mg'=cond(_n==_N,sum(`mg'),.) if `es'
			gen `vtyp' `varn' = `mg' if `touse' 
			label var `varn' "cum. Martingale-like resid."
			exit
		}
	}



		/* Step 9:
			handle switch options that can be used in-sample only.
			Same comments as for step 8.
		*/
	* qui replace `touse'=0 if !e(sample)


			/* Step 10.
				Issue r(198), syntax error.
				The user specified more than one option
			*/
	error 198
end
